<?php
/*
Plugin Name: CAS Authentication
Version: 2.5.1
Plugin URI: Not public
Description: This plugin is a modification of the plugin developed by  Yuta Sakurai http://github.com/sakuraiyuta/wordpress-casi Adding UBC Specific capabilities (Legacy hash checking and PUID identifiers.
Author:  Yuta Sakurai modifified for UBC by Scott McMillan
Author URI: http://ctlt.ubc.ca
License: GPLv2
*/

/* Copyright (C) 2010 Yuta Sakurai <sakurai.yuta@gmail.com>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */


if ( defined( 'XMLRPC_REQUEST' ) ) {
	return;
}

if ( defined( 'WP_CLI' ) && WP_CLI ) {
	return;
}


if ( isset( $_POST['custom_lmslogin'] ) && 'bypassCWL4BasicLTI' == $_POST['custom_lmslogin'] ) {
	require_once( WPMU_PLUGIN_DIR.'/IMSBasicLTI.php' );
	return;
}

include_once( 'cas-authentication/legacy-cwl-unique-id-hash.php' );

global $cas_authentication_opt;
$cas_authentication_opt = get_site_option( 'cas_authentication_options' );

if ( ! $cas_authentication_opt || empty( $cas_authentication_opt ) ) {
	$cas_authentication_opt = get_option( 'cas_authentication_options' );
}

// Overwrite the php path with the wp-config define if it is set
if ( defined( 'UBC_CAS_PHP_PATH' ) ) {
	$cas_authentication_opt['include_path'] = UBC_CAS_PHP_PATH;
}

global $cas_configured;
$cas_configured = true;

// main CAS.php file resides at WP_CONTENT_DIR . /mu-plugins/cas-authentication/cas-authentication/ubc_cas_pkg/CAS.php
// However, we may want to override that in CAS options, so check that first, if it's empty, include the CAS file in this release
if ( ! isset( $cas_authentication_opt['include_path'] ) || empty( $cas_authentication_opt['include_path'] ) ) {

	// No option set in NA dashboard, so use the file in this plugin
	include_once WP_CONTENT_DIR . '/mu-plugins/cas-authentication/cas-authentication/ubc_cas_pkg/CAS.php';

} else {

	// Looks like a file path is set in the NA plugin settings, try to include it
	if ( file_exists( $cas_authentication_opt['include_path'] ) ) {
		include_once $cas_authentication_opt['include_path'];
	} else {
		$cas_configured = false;
	}
}


// Allow us to add a wp-config define for server_hostname, server_path and server_port
if ( defined( 'UBC_CAS_HOSTNAME' ) ) {
	$cas_authentication_opt['server_hostname'] = UBC_CAS_HOSTNAME;
}

if ( defined( 'UBC_CAS_SERVER_PATH' ) ) {
	$cas_authentication_opt['server_path'] = UBC_CAS_SERVER_PATH;
}

if ( defined( 'UBC_CAS_SERVER_PORT' ) ) {
	$cas_authentication_opt['server_port'] = UBC_CAS_SERVER_PORT;
}


if ( '' == $cas_authentication_opt['server_hostname'] || '' == $cas_authentication_opt['server_path'] || 0 == intval( $cas_authentication_opt['server_port'] ) ) {
	$cas_configured = false;
}

if ( $cas_configured ) {
	//phpCAS::client($cas_authentication_opt['cas_version'],
	/* CTLT ADD */
	// hardcoded the SAML client version we only want SAML here.
	phpCAS::client( SAML_VERSION_1_1, $cas_authentication_opt['server_hostname'], intval( $cas_authentication_opt['server_port'] ), $cas_authentication_opt['server_path'] );
	/* CTLT END */

	// function added in phpCAS v. 0.6.0
	// checking for static method existance is frustrating in php4
	$phpCas = new phpCas();

	if ( method_exists( $phpCas, 'setNoCasServerValidation' ) ) {
		phpCAS::setNoCasServerValidation();
		unset( $phpCas );
		// if you want to set a cert, replace the above few lines
	}
}

// So we'll add the hooks on a function
add_action( 'plugins_loaded', 'cas_authentication_add_hooks' );

/**
 * Add all of the object hooks. Really, this should be in the main class, but
 * without changing a fair chunk of stuff, it's easier to leave them outside for now
 *
 * @since 1.0.0
 *
 * @param null
 * @return null
 */

function cas_authentication_add_hooks() {

	// We only want to do this if we're not local - should allow us to do some testing
	if ( ! defined( 'CTLT_ENVIRONMENT' ) || 'LOCAL' === CTLT_ENVIRONMENT ) {
		return;
	}

	// plugin hooks into authentication system
	add_filter( 'wp_authenticate', array( 'CASAuthentication', 'authenticate' ), 10, 3 );
	add_action( 'wp_logout', array( 'CASAuthentication', 'logout' ), 20 );
	add_action( 'lost_password', array( 'CASAuthentication', 'disable_function' ) );
	add_action( 'retrieve_password', array( 'CASAuthentication', 'disable_function' ) );
	add_action( 'password_reset', array( 'CASAuthentication', 'disable_function' ) );
	add_filter( 'check_password', array( 'CASAuthentication', 'override_password_check' ), 10, 4 );
	add_action( 'lost_password', array( 'CASAuthentication', 'disable_function' ) );
	add_filter( 'show_password_fields', array( 'CASAuthentication', 'show_password_fields' ) );
	add_filter( 'login_url', array( 'CASAuthentication', 'bypass_reauth' ), 1 );
	add_action( 'login_enqueue_scripts', array( 'CASAuthentication', 'login_enqueue_scripts__enqueue_css' ) );

}/* cas_authentication_add_hooks() */

$redirect_to = '';
if ( isset( $_GET['redirect_to'] ) ) {
	 $redirect_to = sanitize_text_field( $_GET['redirect_to'] );
}

if ( ! class_exists( 'CASAuthentication' ) ) {

	class CASAuthentication {

		protected static $correct_null_user = false;

		protected static $userID = false;

		// password used by the plugin
		function passwordRoot() {
			return 'Authenticated through CAS';
		}

		/*
		We call phpCAS to authenticate the user at the appropriate time
		(the script dies there if login was unsuccessful)
		If the user has not logged in previously, we create an accout for them
		*/

		function authenticate( &$username, &$password ) {

			global $wpdb, $using_cookie, $cas_authentication_opt, $cas_configured, $redirect_to;

			// We can get here from an admin redirect, so let's check if someone is already logged in
			$userID = get_current_user_ID();
			self::$userID = $userID;

			if ( ! $cas_configured ) {
				die( 'cas-authentication plugin not configured' );
			}

			// Reset values from input ($_POST and $_COOKIE)
			$username = $password = '';

			$headers_all = getallheaders();

			// This hacky bit is get this thing to work with the load balancer.
			if ( isset( $headers_all['X_FORWARDED_PROTO'] ) && 'https' === $headers_all['X_FORWARDED_PROTO']  ) {
				$_SERVER['HTTP_X_FORWARDED_PROTO'] = 'https';
			}

			if ( isset( $_SERVER['HTTP_X_FORWARDED_PROTO'] ) && 'https' === $_SERVER['HTTP_X_FORWARDED_PROTO'] ) {
				$_SERVER['HTTPS'] = 'on';
				$_SERVER['SERVER_PORT'] = '443';
			}

			phpCAS::forceAuthentication();

			// might as well be paranoid
			if ( ! phpCAS::isAuthenticated() ) {
				wp_die( __( 'CAS Authentication Failure. Please visit http://sites.olt.ubc.ca/wp-content/mu-plugins/cas-authentication/ubc_cas_pkg/ubc_cas.php and send the information presented to lt.hub@ubc.ca' ) );
				exit();
			}

			/* CTLT START  get SAML attributes */
			foreach ( phpCAS::getAttributes() as $key => $value ) {

				if ( 'givenName' == $key ) {
					$first_name = "$value";
				}

				if ( 'sn' == $key ) {
					$last_name = "$value";
				}

				if ( 'studentNumber' == $key ) {
					$role_ids[]  = "$value";
					$ubc_student_role = 'student';
				}

				if ( 'employeeNumber' == $key ) {
					$role_ids[]  = "$value";
					$ubc_employee_role   = 'employee';
				}

				if ( 'puid' == $key ) {
					$puid  = "$value";
				}

				if ( 'mail' == $key ) {
					$mail  = "$value";
				}

				if ( 'ubcAffiliation' == $key ) {
					$ubcAffiliation = "$value";
				}
			}

			if ( empty( $ubcAffiliation ) && 'all' != $cas_authentication_opt['allowed_roles'] ) {
				wp_die( __( 'CAS Authentication: This service is only available to employees and students.' ) );
			}

			if ( empty( $puid ) ) {
				wp_die( __( 'CAS Authentication: No PUID returned. Please visit http://sites.olt.ubc.ca/wp-content/mu-plugins/cas-authentication/ubc_cas_pkg/ubc_cas.php and send the information presented to lt.hub@ubc.ca' ) );
			}

			// First Check if the user has an account with an associated PUID
			$puid_check = '';
			$user_id = get_id_from_puid( $puid );
			self::$userID = $user_id;

			if ( empty( $user_id ) ) {

				if ( ! empty( $role_ids ) ) {
					$user_id = get_userid_by_hashed( $role_ids );
					$hashed_id_check = $user_id;
				}

				// If the user ID is not empty now update the user_meta with the puid
				if ( ! empty( $user_id ) ) {
					$add_user_puid = add_user_meta( $user_id, 'puid', $puid );

					if ( false == $add_user_puid ) {
						wp_die( __( 'CAS Authentication: FAIL unable to update user PUID contact site admin. Please visit http://sites.olt.ubc.ca/wp-content/mu-plugins/cas-authentication/ubc_cas_pkg/ubc_cas.php and send the information presented to lt.hub@ubc.ca' ) );
					}
				}
			}

			$password = md5( CASAuthentication::passwordRoot() );

			if ( ! empty( $user_id ) ) {

				// user already exists get the username and set it.
				$user_info = get_userdata( $user_id );
				$username = $user_info->user_login;
				//$password = $user_info->user_pass;
				$usernameCAS = phpCAS::getUser();

				// UserID should be set so authernticate.
				if ( isset( $user_id ) && isset( $username ) && true != is_user_logged_in() ) {

					CASAuthentication::sign_on_wp_user( $user_id, $username );
					CASAuthentication::redirect_after_login( $redirect_to );

				} else {

					CASAuthentication::redirect_after_login( $redirect_to );

				}
			} else {

				// first time logging in
				$password = md5( CASAuthentication::passwordRoot() );
				$username = phpCAS::getUser();

				if ( 1 == $cas_authentication_opt['new_user'] ) {

					// auto-registration is enabled
					// User is not in the WordPress database
					// they passed CAS and so are authorized
					// add them to the database

					$user_email = '';

					// RT: What is this first conditional? The } on 240 closes if( $cas_authentication_opt['new_user'] == 1 )
					if ( $cas_authentication_opt['email_suffix'] != '' ) {

						if ( ! isset( $mail ) ) {
							$user_email = $username . '@' . $cas_authentication_opt['email_suffix'];
						} else {
							$user_email = $mail;
						}
					}
					$user_info = array();

					// Sanitize some of the fields
					$username = sanitize_user( $username, true );
					$user_email = sanitize_email( $user_email );
					$first_name = sanitize_user( $first_name, true );
					$last_name = sanitize_user( $last_name, true );
					$user_info['user_login'] = $username;
					$user_info['user_pass'] = $password;
					$user_info['user_email'] = $user_email;
					$user_info['user_nicename'] = "$first_name $last_name";
					$user_info['display_name'] = "$first_name $last_name";
					$user_info['first_name'] = "$first_name";
					$user_info['last_name'] = "$last_name";
					$user_id = wp_insert_user( $user_info );

					if ( is_wp_error( $user_id ) ) {
						wp_die( __( 'CAS Authentication: There was an error creating the user. Please visit http://sites.olt.ubc.ca/wp-content/mu-plugins/cas-authentication/ubc_cas_pkg/ubc_cas.php and send the information presented to lt.hub@ubc.ca' ) );
					} else {

						if ( '1' != $user_id ) {
							$ubcAffiliation = preg_replace( '/[^A-Za-z0-9 ]/', '', $ubcAffiliation );

							/* CTLT START add SAML attributes to user_meta */
							add_user_meta( $user_id, 'puid', $puid );

							// Add the UBC Role mainly just for stats if we ever use it....
							add_user_meta( $user_id, 'ubcAffiliation', "$ubcAffiliation" );

							CASAuthentication::sign_on_wp_user( $user_id, $username );
							CASAuthentication::redirect_after_login( $redirect_to );
							/* CTLT END */
						}
					}
				} // Close CAS Auth new user

			}

		}/* authenticate() */

		public function set_current_user__correctNullUser() {

			global $current_user;
			if ( ! self::$correct_null_user || 0 !== $current_user->ID ) {
				return;
			}

			$real_user = self::$correct_null_user;

			// Ensure we don't get stuck in an infinite loop here
			self::$correct_null_user = false;
			wp_set_current_user( $real_user );

		}/* set_current_user__correctNullUser() */

		/**
		 *  redirect_after_login if redirect_to set push there if not admin_url
		 *
		 *  @param string $redirect_to
		 */

		function redirect_after_login( $redirect_to ) {

			if ( ! isset( $redirect_to ) ) {
				wp_redirect( network_home_url() );
				exit();
			}

			// Check if redirecting to a domain mapped url
			global $wpdb, $current_blog;
			$dmTable = $wpdb->base_prefix . 'domain_mapping';

			// Prepare and sanitize the redirect to
			$rawUrl = esc_url_raw( $redirect_to );

			$rawUrl = str_replace( 'https://', '', $rawUrl );
			$rawUrl = str_replace( 'http://', '', $rawUrl );

			// We only want the domain, so if there's another / strip it and anything after it
			$rawUrl = htmlspecialchars( static::urlToDomain( $rawUrl ) );

			// if this is blank, run away
			if ( ! $rawUrl || empty( $rawUrl ) ) {
				wp_redirect( $redirect_to );
				exit();
			}

			// Now look for this in the domain mapping table
			$blogID = $wpdb->get_var( "SELECT blog_id FROM $dmTable WHERE domain = '$rawUrl' AND active = '1' LIMIT 1" );

			// If we don't have a blog ID, it's not a mapped domain, so just go with what was passed in
			if ( ! $blogID ) {

				if ( false == wp_validate_auth_cookie() && is_user_logged_in() ) {

					// wp_set_auth_cookie( get_current_user_id(), true, false ); // RT: Removed this as it seems unnecessary.
					wp_set_auth_cookie( get_current_user_id(), true, true );

				}

				wp_redirect( $redirect_to );
				exit();

			}

			// Looks like we have a blog ID, so we need to get the original URL for it
			$originalUrl = apply_filters( 'ubc_ee_get_original_url',  get_original_url( 'siteurl', $blogID ) );

			if ( static::stringEndsWithString( $redirect_to, 'wp-admin' ) || static::stringEndsWithString( $redirect_to, 'wp-admin/' ) ) {

				wp_redirect( trailingslashit( $originalUrl ) . 'wp-admin/' );
				exit();

			}

			wp_redirect( trailingslashit( $originalUrl ) );
			exit();

		}/* redirect_after_login() */


		/**
		 *  sign_on_wp_user  -  after login
		 *
		 *  @param string $user_id
		 *  @param string $username
		 *  return $user
		*/

		function sign_on_wp_user( $user_id, $username ) {

			if ( false == wp_validate_auth_cookie() ) {
				wp_set_auth_cookie( $user_id, true, false );
			}

			return new WP_User( $user_id );

		}/* sign_on_wp_user() */

		/*
		 * We use the provided logout method
		 */
		function logout() {

			global $cas_configured;

			if ( ! $cas_configured ) {
				die( 'cas-authentication not configured' );
			}

			wp_destroy_current_session();

			// Akin to wp_destroy_all_sessions()
			$uid = get_current_user_id();
			$manager = WP_Session_Tokens::get_instance( $uid );
			$manager->destroy_all();

			wp_clear_auth_cookie();
			wp_set_current_user( 0 );

			session_unset();
			session_destroy();

			if ( 'logout' == sanitize_text_field( $_GET['action'] ) ) {

				$redirectTo = ( isset( $_GET['redirect_to'] ) ) ? $_GET['redirect_to'] : false;

				// If there's no redirect, it's just back to the network site, otherwise it's the network site with a redirect_to
				if ( $redirectTo ) {
					phpCAS::logoutWithRedirectService( network_site_url( null, 'http' ) . '?dmlogout=true&redirect_to=' . esc_url_raw( $redirectTo ) );
				} else {
					phpCAS::logoutWithRedirectService( network_site_url( null, 'http' ) . '?dmlogout=true' );
				}
			}

			exit();

		}/* logout() */

		/*
		* Remove the reauth=1 parameter from the login URL, if applicable. This allows
		* us to transparently bypass the mucking about with cookies that happens in
		* wp-login.php immediately after wp_signon when a user e.g. navigates directly
		* to wp-admin.
		*/
		function bypass_reauth( $login_url ) {

			/**
			  *In order for CAS to work, we need to hit the *root* site url for the wp-login.php. i.e. get_site_url(1) . '/wp-login.php';
			  *BUT
			  *if there is a ?redirect_to in the $login_url we need to append that to the above url. (and, we also need to remove reauth)
			  *We need to do some jiggerypokery with the $login_url string to get out the redirect_to
			**/

			$redirectTo = false;

			// We need to strip this from $login_url;
			$thisSiteURL = get_site_url();

			// Possible mixed protocols, so strip them out, but only from the main login url, not the redirect
			$protocolAgnosticLoginURL = static::replaceFirst( $login_url, 'https://', '' );
			$protocolAgnosticLoginURL = static::replaceFirst( $protocolAgnosticLoginURL, 'http://', '' );

			$protocolAgnosticSiteURL = static::replaceFirst( $thisSiteURL, 'https://', '' );
			$protocolAgnosticSiteURL = static::replaceFirst( $protocolAgnosticSiteURL, 'http://', '' );

			$pos = strpos( $protocolAgnosticLoginURL, $protocolAgnosticLoginURL );

			// If there's something to strip out, then do it
			if ( false !== $pos ) {

				// strip the site url from the login url
				$loginURLWithoutSite = static::replaceFirst( $protocolAgnosticLoginURL, $protocolAgnosticSiteURL, '' );

				// strip everything before the '?'
				$justQueryArgs = strstr( $loginURLWithoutSite, '?' );

				if ( false !== $justQueryArgs ) {

					// Get a URL with all of the query args
					$queryArgs = array();

					parse_str( urldecode( $justQueryArgs ), $queryArgs );

					// If we have a redirect_to, let's make sure we add it later
					if ( array_key_exists( '?redirect_to', $queryArgs ) ) {
						$redirectTo = $queryArgs['?redirect_to'];
					}
				}
			}

			// Root site's login url
			$login_url = get_site_url( '1', null, 'https' ) . '/wp-login.php';

			// This was from the original method, I'm not sure it's actually doing anything... RT
			$login_url = remove_query_arg( 'reauth', $login_url );

			// If we have a redirect, re-append it
			if ( false !== $redirectTo ) {
				$login_url = add_query_arg( 'redirect_to', $redirectTo, $login_url );
			}

			// Ship
			return $login_url;

		}/* bypass_reauth() */

		/**
		 * After login, redirect as necessary
		 *
		 * @author Richard Tape <@richardtape>
		 * @since 1.0
		 * @param
		 * @return
		 */

		function login_redirect__redirectIfNecessary( $redirect_to, $request, $user ) {

			return $redirect_to;

		}/* login_redirect__redirectIfNecessary() */


		/**
		 * Replace first instance of substring in string
		 *
		 * @since 0.1
		 *
		 * @param string $input haystack
		 * @param string $search needle
		 * @param string $replacement replacement
		 * @return string the string with the first copy of $search removed if it exists
		 */

		public static function replaceFirst( $input, $search, $replacement ) {

			$pos = stripos( $input, $search );

			if ( false === $pos ) {
				return $input;
			} else {
				$result = substr_replace( $input, $replacement, $pos, strlen( $search ) );
				return $result;
			}

		}/* replaceFirst() */

		/**
		 * Break a full url down into just it's domain name
		 *
		 * @author Richard Tape <@richardtape>
		 * @since 1.0
		 * @param (string) $url - the full url we just want the domain for
		 * @return (string) $host - the domain
		 */

		public static function urlToDomain( $url ) {

			$host = @parse_url( $url, PHP_URL_HOST );

			// If the URL can't be parsed, use the original URL
			// Change to "return false" if you don't want that
			if ( ! $host ) {
				$host = $url;
			}

			// The "www." prefix isn't really needed if you're just using
			// this to display the domain to the user
			if ( 'www.' == substr( $host, 0, 4 ) ) {
				$host = substr( $host, 4 );
			}

			$host = explode( '/', $host, 2 );

			return $host[0];

		}/* url_to_domain() */

		public static function stringEndsWithString( $string, $test ) {

			$strlen = strlen( $string );
			$testlen = strlen( $test );
			if ( $testlen > $strlen ){
				return false;
			}

			return substr_compare( $string, $test, $strlen - $testlen, $testlen ) === 0;

		}/* stringEndsWithString() */

		/*
		* Don't show password fields on user profile page.
		*/
		function show_password_fields( $show_password_fields ) {
			return false;
		}

		function disable_function() {
			die('Disabled');
		}

		function override_password_check( $check, $password, $hash, $user_id ) {
			return $check;
		}

		/**
		 * Register and enqueue our CSS to hide the form and display a message
		 *
		 * @since 1.0.0
		 *
		 * @param null
		 * @return null
		 */

		public function login_enqueue_scripts__enqueue_css() {

			?>
			<style>
			#login {
				display: none;
			}
			.logout-message-bar {
				background: white;
				padding: 20px;
				margin: 20px auto 20px;
				max-width: 1160px;
			}
			.logout-message-bar i {
				font-size: 6em;
				float: left;
				margin-right: 20px;
				position: relative;
				top: -10px;
			}
			.logout-message-bar h2 {
				margin: 0;
			}
			</style>

			<div class="logout-message-bar" id="logout-message-bar">
				<i class="icon-exclamation-sign"></i>
				<h2>You are logged out</h2>
				<p>Please close your browser window to ensure that you have logged out completely.</p>
			</div>

			<?php

		}/* login_enqueue_scripts__enqueue_css() */

	}/* class CASAuthentication */

}/* class_exists CASAuthentication */

//----------------------------------------------------------------------------------------
//              SOME FUNCTIONS TO DEAL WITH UBC SPECIFIC / LEGACY STUFF
//---------------------------------------------------------------------------------------

/*
 * Check it the user is in the system alread. Looking for valid PUID or legacy hashed.
 */

function get_id_from_puid( $puid ) {

	global $wpdb;

	$result = $wpdb->get_row( $wpdb->prepare( "SELECT user_id FROM $wpdb->usermeta WHERE meta_key = 'puid' AND meta_value = '%s'", $puid ), ARRAY_A );

	if ( null != $result ) {
		return $result['user_id'];
	}

}/* get_id_from_puid() */

/**
* get_userdata_by_cwl_unique_id - get WordPress user data using CWL unique ID
*
* @param string $CWLUniqueID - CWL unique ID
* @return WordPress user data or NULL if user is not found
*/
function get_userid_by_hashed( $role_ids ) {

	global $wpdb;

	foreach ( $role_ids as $role ) {

		$ID = cwl_salt_hash( $role );

		$result = $wpdb->get_row( $wpdb->prepare( "SELECT user_id FROM $wpdb->usermeta WHERE meta_key = 'CWLUniqueID' AND meta_value = '%s'", $ID ), ARRAY_A );

		if ( null != $result ) {
			return $result['user_id'];
		}
	}

}/* get_userid_by_hashed() */


add_filter( 'allowed_redirect_hosts', 'allowed_redirect_hosts__addRedirectToToHostsWhitelist', 999, 2 );

/**
 * Enable Domain Mapping to log in to mapped domain via adding the redirect-to to the allowed whitelist in
 * wp_validate_redirect()
 *
 * @author Richard Tape <@richardtape>
 * @since 1.0
 * @param (array) $list - existing whitelist
 * @param (string) $location - the location which we wish to add
 * @return (array) $list - modified list
 */

function allowed_redirect_hosts__addRedirectToToHostsWhitelist( $list, $location ) {

	$list[] = $location;

	return $list;

}/* allowed_redirect_hosts__addRedirectToToHostsWhitelist() */

if ( ! function_exists( 'wp_sanitize_redirect' ) ) {

	// override wp core function to preserve @ in URL for CWL
	// CWL ticket has two @ at the end
	function wp_sanitize_redirect($location) {

		$location = preg_replace( '|[^a-z0-9-~+_.?#=&;,/:%!@]|i', '', $location );
		$location = wp_kses_no_null( $location );

		// remove %0d and %0a from location
		$strip = array( '%0d', '%0a', '%0D', '%0A' );
		$location = _deep_replace( $strip, $location );
		return $location;

	}/* wp_sanitize_redirect() */

}/* function_exists( 'wp_sanitize_redirect' ) */

//----------------------------------------------------------------------------
//		ADMIN OPTION PAGE FUNCTIONS
//----------------------------------------------------------------------------

add_action( 'network_admin_menu', 'cas_authentication_add_options_page' );

function cas_authentication_add_options_page() {

	if ( function_exists( 'add_options_page' ) ) {
		add_options_page( 'CAS Authentication', 'CAS Authentication', 'manage_network', 'cas_authentication_options_page', 'cas_authentication_options_page' );
	}

	add_submenu_page( 'settings.php', 'CAS Authentication', 'CAS Authentication', 'manage_sites', 'cas_authentication_options_page', 'cas_authentication_options_page' );

}/* cas_authentication_add_options_page() */


function cas_authentication_options_page() {

	global $wpdb;

	// Setup Default Options Array
	$optionarray_def = array(
		'new_user' => false,
		//'new_blog' => false,
		'register_existing_blog' => false,
		'register_blog_ids' => '1',
		'register_blog_role' => '',
		//'hash_domain' => FALSE,
		'allowed_roles' => 'all',
		'redirect_url' => '',
		//'blog_prefix' => 'cas_',
		'email_suffix' => 'yourschool.edu',
		'cas_version' => SAML_VERSION_1_1,
		'include_path' => '',
		'server_hostname' => 'yourschool.edu',
		'server_port' => '443',
		'server_path' => '',
	);

	if ( isset( $_POST['submit'] ) ) {

		if ( ! wp_verify_nonce( $_POST['_wpnonce'],'_wpnonce' ) ) {
			wp_die( __( 'Error: nonce did not match.' ) );
		}

		// Options Array Update
		$optionarray_update = array(
			'new_user' => $_POST['new_user'],
			//'new_blog' => $_POST['new_blog'],
			'register_existing_blog' => sanitize_text_field( $_POST['register_existing_blog'] ),
			'register_blog_ids' => sanitize_text_field( $_POST['register_blog_ids'] ),
			'register_blog_role' => sanitize_text_field( $_POST['register_blog_role'] ),
			//'hash_domain' => $_POST['hash_domain'],
			'allowed_roles' => sanitize_text_field( $_POST['allowed_roles'] ),
			'redirect_url' => sanitize_text_field( $_POST['redirect_url'] ),
			//'blog_prefix' => $_POST['blog_prefix'],
			'email_suffix' => sanitize_text_field( $_POST['email_suffix'] ),
			'include_path' => sanitize_text_field( $_POST['include_path'] ),
			'cas_version' => sanitize_text_field( $_POST['cas_version'] ),
			'server_hostname' => sanitize_text_field( $_POST['server_hostname'] ),
			'server_port' => sanitize_text_field( $_POST['server_port'] ),
			'server_path' => sanitize_text_field( $_POST['server_path'] ),
		);

		update_site_option( 'cas_authentication_options', $optionarray_update );

	}

	// Get Options
	$optionarray_def = get_site_option( 'cas_authentication_options' );

	global $current_site, $wpdb;

	$editblog_roles = get_blog_option(
		$current_site->id,
		$wpdb->get_blog_prefix( $current_site->id ) . 'user_roles'
	);

	$network_admin_url = network_admin_url();

	?>

	<div class="wrap">

		<h2>CAS Authentication Options</h2>

		<div id="message" class="error">
			<h4>Caution</h4>
			<p> If you want to use <em>Auto-register new blogs</em>, <strong>you must set &quot;Network Activate&quot; in Plugins page, not use &quot;Activate&quot;!</strong></p>
		</div>

		<form method="post" action="<?php echo esc_url( $network_admin_url ) . '?page=cas_authentication_options_page'?>&updated=true">

			<fieldset class="options">

				<h3>User registration options</h3>

				<table class="form-table">

					<tbody>

						<tr>
							<th scope="row"><label for="new_user_inp">Auto-register new users?</label></th>
							<td>
								<input name="new_user" type="checkbox" id="new_user_inp" value="1" <?php checked( '1', $optionarray_def['new_user'] ); ?> />
								<p class="description" id="">Checking <em>Auto-register new users</em> will automatically create a new user (with role of Subscriber) upon successful login of a new visitor to the site.</p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="">To whom is this service available?</label></th>
							<td>
								<fieldset>
									<legend class="screen-reader-text">Service availability settings</legend>
									<label><input name="allowed_roles" type="radio" id="allowed_roles" value="employee_student" <?php checked( 'employee_student', $optionarray_def['allowed_roles'] ); ?> /> Employees / Students</label><br />
									<label><input name="allowed_roles" type="radio" id="allowed_roles" value="all" <?php checked( 'all', $optionarray_def['allowed_roles'] ); ?> /> All Roles (including basic)</label>
								</fieldset>
							</td>
						</tr>


						<tr>
							<th scope="row"><label for="register_existing_blog_inp">Auto-register new cas-user for existing blogs?</label></th>
							<td>
								<input name="register_existing_blog" type="checkbox" id="register_existing_blog_inp" value="1" <?php checked( '1', $optionarray_def['register_existing_blog'] ); ?> />
								<p class="description" id="">Checking <em>Auto-register new cas-user for existing blogs</em> will automatically register new cas-user for existing blog upon successful login of a new visitor to the site.</p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="register_blog_ids_inp">Automatic registration Site IDs</label></th>
							<td>
								<input name="register_blog_ids" type="text" id="register_blog_ids_inp" value="<?php echo esc_html( ( ! count( explode( ',', $optionarray_def['register_blog_ids'] ) ) ) ? '' : $optionarray_def['register_blog_ids'] ); ?>" />
								<p class="description" id="">Please insert blog-ids sepalated by comma. (ex: 1,5,10)</p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="register_blog_role">Default role</label></th>
							<td>
								<select name="register_blog_role" type="checkbox" id="register_blog_role">
									<?php foreach ( $editblog_roles as $role => $role_assoc ) : ?>
										<?php $name = translate_user_role( $role_assoc['name'] ); ?>
										<option <?php selected( $role, $optionarray_def['register_blog_role'] ); ?> value="<?php echo esc_attr( $role ); ?>"><?php echo esc_html( $name ); ?></option>
									<?php endforeach; ?>
								</select>
								<p class="description" id="">Users automatically added to the previous sites will be given this role by default</p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="email_suffix_inp">E-mail Suffix</label></th>
							<td>
								netid@<input type="text" name="email_suffix" id="email_suffix_inp" value="<?php echo esc_html( $optionarray_def['email_suffix'] ); ?>" size="35" />
								<p class="description" id="">If you know that the owner of the CAS authentication service issues email addresses based on their netids, you can predict your users' emails here</p>
							</td>
						</tr>

					</tbody>

				</table>

				<h3>phpCAS options</h3>
				<p>Note: Once you fill in these options, wordpress authentication will happen through CAS, even if you misconfigure it. To avoid being locked out of Wordpress, use a second browser to check your settings before you end this session as administrator. If you get an error in the other browser, correct your settings here. If you can not resolve the issue, disable this plug-in.</p>

				<table class="form-table">

					<tbody>

						<tr>
							<th scope="row"><label for="include_path_inp">CAS.php script location</label></th>
							<td>
								<?php
									$casScriptConfig = ( defined( 'UBC_CAS_PHP_PATH' ) ) ? UBC_CAS_PHP_PATH : false;
									$casScriptValue = ( $casScriptConfig ) ? $casScriptConfig : $optionarray_def['include_path'];
									$casScriptDisabled = ( $casScriptConfig ) ? ' disabled="disabled"' : '';
									$casScriptMessage = ( $casScriptConfig ) ? '<strong>Note: This setting is currently set in wp-config. You can not edit it on this settings screen.</strong> ' : '';
								?>
								<input type="text" name="include_path" id="include_path_inp" value="<?php echo esc_html( $casScriptValue ); ?>" class="large-text" <?php echo esc_html( $casScriptDisabled ); ?> />
								<p class="description" id=""><?php esc_html_e( $casScriptMessage ); ?>Full absolute path to CAS.php script. (optional. If left blank, CAS.php file from the CAS plugin will be used)</p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="">CAS version</label></th>
							<td>
								<p>SAML_VERSION_1_1</p>
								<p class="description" id="">For UBC this is hard-coded for SAML 1.1</p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="server_hostname_inp">Server Hostname</label></th>
							<td>
								<?php
									$casScriptConfig = ( defined( 'UBC_CAS_HOSTNAME' ) ) ? UBC_CAS_HOSTNAME : false;
									$casScriptValue = ( $casScriptConfig ) ? $casScriptConfig : $optionarray_def['server_hostname'];
									$casScriptDisabled = ( $casScriptConfig ) ? ' disabled="disabled"' : '';
									$casScriptMessage = ( $casScriptConfig ) ? '<strong>Note: This setting is currently set in wp-config. You can not edit it on this settings screen.</strong> ' : '';
								?>
								<input type="text" name="server_hostname" id="server_hostname_inp" value="<?php echo esc_html( $casScriptValue ); ?>" <?php echo esc_html( $casScriptDisabled ); ?> />
								<p class="description" id=""><?php esc_html_e( $casScriptMessage ); ?></p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="server_port_inp">Server Port</label></th>
							<td>
								<?php
									$casScriptConfig = ( defined( 'UBC_CAS_SERVER_PORT' ) ) ? UBC_CAS_SERVER_PORT : false;
									$casScriptValue = ( $casScriptConfig ) ? $casScriptConfig : $optionarray_def['server_port'];
									$casScriptDisabled = ( $casScriptConfig ) ? ' disabled="disabled"' : '';
									$casScriptMessage = ( $casScriptConfig ) ? '<strong>Note: This setting is currently set in wp-config. You can not edit it on this settings screen.</strong> ' : '';
								?>
								<input type="text" name="server_port" id="server_port_inp" value="<?php echo esc_html( $casScriptValue ); ?>" <?php echo esc_html( $casScriptDisabled ); ?> />
								<p class="description" id=""><?php esc_html_e( $casScriptMessage ); ?></p>
							</td>
						</tr>

						<tr>
							<th scope="row"><label for="server_path_inp">Server Path</label></th>
							<td>
								<?php
									$casScriptConfig = ( defined( 'UBC_CAS_SERVER_PATH' ) ) ? UBC_CAS_SERVER_PATH : false;
									$casScriptValue = ( $casScriptConfig ) ? $casScriptConfig : $optionarray_def['server_path'];
									$casScriptDisabled = ( $casScriptConfig ) ? ' disabled="disabled"' : '';
									$casScriptMessage = ( $casScriptConfig ) ? '<strong>Note: This setting is currently set in wp-config. You can not edit it on this settings screen.</strong> ' : '';
								?>
								<input type="text" name="server_path" id="server_path_inp" value="<?php echo esc_html( $casScriptValue ); ?>" <?php echo esc_html( $casScriptDisabled ); ?> />
								<p class="description" id=""><?php esc_html_e( $casScriptMessage ); ?></p>
							</td>
						</tr>

					</tbody>

				</table>

			</fieldset>

			<p class="submit">
				 <input type="submit" name="submit" class="button button-primary" value="<?php esc_attr_e( 'Update Options' ) ?> &raquo;" />
			</p>

			<?php wp_nonce_field( '_wpnonce' ); ?>

		</form>

	</div>

<?php
}/* cas_authentication_options_page() */

// Add a metabox to the user profile, for super admins only, which shows the PUID
add_action( 'show_user_profile', 'cas_authentication_add_puid_profile_field' );
add_action( 'edit_user_profile', 'cas_authentication_add_puid_profile_field' );

function cas_authentication_add_puid_profile_field( $user ) {

	if ( ! is_super_admin() ) {
		return;
	}

	?>

	<h3>PUID</h3>

	<table class="form-table">

		<tr>
			<th><label for="puid">PUID</label></th>

			<td>
				<input type="text" name="puid" id="puid" value="<?php echo esc_attr( get_the_author_meta( 'puid', $user->ID ) ); ?>" class="regular-text" /><br />
				<span class="description">User's PUID.</span>
			</td>
		</tr>

	</table>
<?php }

add_action( 'personal_options_update', 'cas_authentication_save_puid_field' );
add_action( 'edit_user_profile_update', 'cas_authentication_save_puid_field' );

function cas_authentication_save_puid_field( $user_id ) {

	if ( ! is_super_admin() ) {
		return;
	}

	update_usermeta( $user_id, 'puid', esc_html( $_POST['puid'] ) );
}


add_action( 'update_user_meta', 'ubc_fix_user_session_ballooning', 1, 4 );

/**
 * We don't want people having a bajillion user sessions. Not only is that stupid,
 * it probably slows down login, and also if they're stuck in a loop, it can
 * take down CMS because of our MySQL binlogs. This method attempts to catch
 * whenever a new session_token is created that we do some sanity checks.
 *
 * If a user has more than 50 sessions, that seems silly, so we boot them,
 * remove all their sessions and site cookies and redirect them. If a user has
 * more than 25 sessions and 5 or more of them have the same login timestamp
 * then it seems like the user is in a loop. So we boot then, clear all sessions
 * and cookies, and redirect them. Super difficult to test.
 *
 * @since 1.0.0
 *
 * @param null
 * @return null
 */

function ubc_fix_user_session_ballooning( $meta_id, $object_id, $meta_key, $_meta_value ) {

	if ( 'session_tokens' !== $meta_key ) {
		return;
	}

	if ( ! is_array( $_meta_value ) ) {
		return;
	}

	// First test; Have over 50 sessions? Clear 'em all, boot the user, redirect with logout
	if ( count( $_meta_value ) > 50 ) {
		ubc_clear_all_sessions_and_logout( $object_id, 'max_sessions' );
	}


	// Second test; have 5 or more session_tokens that have the same 'login' time?
	$timestamps = array();

	$bail = false;

	if ( count( $_meta_value ) > 25 ) {
		foreach ( $_meta_value as $key => $session_data ) {
			$timestamps[ $session_data['login'] ][] = $session_data['login'];
			if ( count( $timestamps[ $session_data['login'] ] ) >= 5 ) {
				$bail = true;
				break;
			}
		}
	}

	if ( true === $bail ) {
		ubc_clear_all_sessions_and_logout( $object_id, 'max_same_sessions' );
	}

}/* ubc_fix_user_session_ballooning() */


/**
 * Destroy sessions for a user. Clear cookies. Clear PHP sessions. Log out of
 * CAS and redirect to CMS home page with a message.
 *
 * @since 1.0.0
 *
 * @param null
 * @return null
 */

function ubc_clear_all_sessions_and_logout( $user_id, $reason = '' ) {

	wp_destroy_current_session();

	// Akin to wp_destroy_all_sessions()
	$manager = WP_Session_Tokens::get_instance( $user_id );
	$manager->destroy_all();

	wp_clear_auth_cookie();
	wp_set_current_user( 0 );

	session_unset();
	session_destroy();

	phpCAS::logoutWithRedirectService( 'http://cms.ubc.ca/?dmlogout=true&session_break=true&reason=' . esc_attr( $reason ) );

	exit();

}/* ubc_clear_all_sessions_and_logout() */
